@@ -17,7 +17,7 @@
 %                                 July 2000                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright @ 2000 ImageMagick Studio LLC, a non-profit organization         %
+%  Copyright @ 1999 ImageMagick Studio LLC, a non-profit organization         %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -286,7 +286,7 @@ static inline MagickBooleanType CompareMagic(const unsigned char *magic,
   return(MagickFalse);
 }
 
-static MagickBooleanType IsMagicCacheInstantiated()
+static MagickBooleanType IsMagicCacheInstantiated(void)
 {
   if (magic_cache == (LinkedListInfo *) NULL)
     {
@@ -472,7 +472,7 @@ static int MagicInfoCompare(const void *x,const void *y)
 #endif
 
 MagickExport const MagicInfo **GetMagicInfoList(const char *pattern,
-  size_t *number_aliases,ExceptionInfo *magick_unused(exception))
+  size_t *number_aliases,ExceptionInfo *exception)
 {
   const MagicInfo
     **aliases;
@@ -489,7 +489,7 @@ MagickExport const MagicInfo **GetMagicInfoList(const char *pattern,
   if (IsEventLogging() != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",pattern);
   *number_aliases=0;
-  if (IsMagicCacheInstantiated() == MagickFalse)
+  if (IsMagicListInstantiated(exception) == MagickFalse)
     return((const MagicInfo **) NULL);
   aliases=(const MagicInfo **) AcquireQuantumMemory((size_t)
     GetNumberOfElementsInLinkedList(magic_list)+1UL,sizeof(*aliases));
@@ -569,7 +569,7 @@ static int MagicCompare(const void *x,const void *y)
 #endif
 
 MagickExport char **GetMagicList(const char *pattern,size_t *number_aliases,
-  ExceptionInfo *magick_unused(exception))
+  ExceptionInfo *exception)
 {
   char
     **aliases;
@@ -586,7 +586,7 @@ MagickExport char **GetMagicList(const char *pattern,size_t *number_aliases,
   if (IsEventLogging() != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",pattern);
   *number_aliases=0;
-  if (IsMagicCacheInstantiated() == MagickFalse)
+  if (IsMagicListInstantiated(exception) == MagickFalse)
     return((char **) NULL);
   aliases=(char **) AcquireQuantumMemory((size_t)
     GetNumberOfElementsInLinkedList(magic_list)+1UL,sizeof(*aliases));
@@ -800,7 +800,7 @@ MagickPrivate MagickBooleanType MagicComponentGenesis(void)
 
 static void *DestroyMagicElement(void *magic_info)
 {
-  RelinquishMagickMemory((MagicInfo *) magic_info);
+  (void) RelinquishMagickMemory((MagicInfo *) magic_info);
   return((void *) NULL);
 }
 
